@@ -1,6 +1,6 @@
 /* $RCSfile: dag.c,v $
--- $Revision: 1.7.2.5 $
--- last change: $Author: vq $ $Date: 2006/11/19 05:27:24 $
+-- $Revision: 1.10.2.1 $
+-- last change: $Author: vq $ $Date: 2007/07/24 23:00:26 $
 --
 -- SYNOPSIS
 --      Routines to construct the internal dag.
@@ -28,8 +28,6 @@
 
 #include "extern.h"
 
-static char *_normalize_path(char *path);
-
 
 static void
 set_macro_value(hp)/*
@@ -42,9 +40,26 @@ HASHPTR hp;
    switch( hp->ht_flag & M_VAR_MASK )	/* only one var type per var */
    {
       case M_VAR_STRING:
-         *hp->MV_SVAR = hp->ht_value;
-         break;
-         
+	 *hp->MV_SVAR = hp->ht_value;
+	 /* Add special treatment for PWD/MAKEDIR for .WINPATH. */
+	 if( hp->MV_SVAR == &Pwd_macval ) {
+	    if( Pwd )
+	       FREE(Pwd);
+	    Pwd = hp->ht_value;
+	    /* Use the "DOSified" path for the macro. */
+	    *hp->MV_SVAR = hp->ht_value = DmStrDup(DO_WINPATH(hp->ht_value));
+	    DB_PRINT( "smv", ("PWD: %s/%s", Pwd_macval, Pwd) );
+	 } else if( hp->MV_SVAR == &Makedir_macval ) {
+	    if( Makedir )
+	       FREE(Makedir);
+	    Makedir = hp->ht_value;
+	    /* Use the "DOSified" path for the macro. */
+	    *hp->MV_SVAR = hp->ht_value = DmStrDup(DO_WINPATH(hp->ht_value));
+	    DB_PRINT( "smv", ("MAKEDIR: %s/%s", Makedir_macval, Makedir) );
+	 }
+	 /* No special treatment for TMD needed. */
+	 break;
+
       case M_VAR_CHAR:
          *hp->MV_CVAR = (hp->ht_value == NIL(char)) ? '\0':*hp->ht_value;
          break;
@@ -98,7 +113,20 @@ HASHPTR hp;
 	    if( (hp->MV_MASK & A_SEQ) && (Max_proc != 1) )
 	       Def_macro( "MAXPROCESS", "1", M_MULTI|M_EXPANDED);
 	 }
-         break;
+
+#if defined(__CYGWIN__)
+	 /* Global .WINPATH change. Only needed for cygwin. */
+	 if(hp->MV_MASK & A_WINPATH) {
+	    UseWinpath = ((Glob_attr&A_WINPATH) != 0);
+	    /* Change MAKEDIR, PWD according to .WINPATH. During
+	     * makefile evaluation this cannot change TMD (it is "."
+	     * and later TMD is set in Make() according to the
+	     * .WINPATH attribute. */
+	    Def_macro( "MAKEDIR", Makedir, M_FORCE | M_EXPANDED );
+	    Def_macro( "PWD", Pwd, M_FORCE | M_EXPANDED );
+	 }
+#endif
+	 break;
    }
 }
 
@@ -263,12 +291,14 @@ HASHPTR hp;
 PUBLIC HASHPTR
 Def_macro( name, value, flags )/*
 =================================
-   This routine is used to define a macro, and it's value.
+   This routine is used to define a macro, and it's value. A copy of
+   the content of value is stored and not the pointer to the value.
    The flags indicates if it is a permanent macro or if it's value
    can be redefined.  A flags of M_PRECIOUS means it is a precious
-   macro and cannot be further redefined.  If the flags flag also
-   contains the M_MULTI bit it means that the macro can be redefined
-   multiple times and no warning of the redefinitions should be issued.
+   macro and cannot be further redefined unless M_FORCE is used.
+   If the flags flag contains the M_MULTI bit it means that the macro
+   can be redefined multiple times and no warning of the redefinitions
+   should be issued.
    Once a macro's VAR flags are set they are preserved through all future
    macro definitions.
    
@@ -327,19 +357,24 @@ int     flags;			/* initial ht_flags	*/
 
    /* If an empty string ("") is given set ht_value to NIL(char) */
    if( (value != NIL(char)) && (*value) ) {
-      /* strip out any \<nl> combinations where \ is the current CONTINUATION
-       * char */ 
-
-      for( p = value; (p = strchr(p, CONTINUATION_CHAR)) != NIL(char); )
-         if( p[1] == '\n' )
-	    strcpy( p, p+2 );
-	 else
-	    p++;
 
       if( !(flags & M_LITERAL) ) {
-	 p = DmStrDup( DmStrSpn(value," \t")); /* strip white space before */
-					       /* ... and after value	   */
-	 if( *p ) {
+	 q = DmStrDup(value);
+	 /* strip out any \<nl> combinations where \ is the current
+	  * CONTINUATION char */
+	 for(p=q; (p=strchr(p,CONTINUATION_CHAR))!=NIL(char); )
+	    if( p[1] == '\n' )
+	       strcpy( p, p+2 );
+	    else
+	       p++;
+
+	 p = DmStrSpn(q ," \t");	/* Strip white space before ... */
+	 if( p != q ) {
+	    strcpy( q, p);
+	    p = q;
+	 }
+
+	 if( *p ) {			/* ... and after the value. */
 	    for(q=p+strlen(p)-1; ((*q == ' ')||(*q == '\t')); q--);
 	    *++q = '\0';
 	 }
@@ -358,16 +393,18 @@ int     flags;			/* initial ht_flags	*/
 
       hp->ht_value = p;
    }
-   else
+   else {
       hp->ht_value = NIL(char);
+      flags |= M_EXPANDED;
+   }
 
    /* Assign the hash table flag less the M_MULTI flag, it is used only
     * to silence the warning.  But carry it over if it was previously
-    * defined in ht_flag, as this is a permanent M_MULTI variable. */
-   /* Also strip M_INIT flag if macro is assigned. */
+    * defined in ht_flag, as this is a permanent M_MULTI variable. Keep
+    * the M_PRECIOUS flag and strip the M_INIT flag. */
 
-   hp->ht_flag = (((flags & ~(M_MULTI|M_FORCE)) |
-		  (hp->ht_flag & (M_VAR_MASK | M_MULTI)))) & ~M_INIT;
+   hp->ht_flag = ((flags & ~(M_MULTI|M_FORCE)) |
+		  (hp->ht_flag & (M_VAR_MASK|M_MULTI|M_PRECIOUS))) & ~M_INIT;
    
    /* Check for macro variables and make the necessary adjustment in the
     * corresponding global variables */
@@ -436,7 +473,7 @@ char    *name;
       DB_PRINT( "path", ("Normalizing [%s]", name) );
 
       /* The normalizing function returns a pointer to a static buffer. */
-      name =_normalize_path(name);
+      name = normalize_path(name);
 
       hp = Get_name( name, Defs, TRUE );/* get the name from hash table */
 
@@ -627,7 +664,12 @@ char *rp;
 	 case '@' : ++atcount;        break;
 	 case '-' : flag |= A_IGNORE; break;
 	 case '+' : flag |= A_SHELL;  break;
-	 case '%' : flag |= A_SWAP;   break;
+	 case '%' :
+#if defined(MSDOS)
+	    /* Ignore % in the non-MSDOS case. */
+	    flag |= A_SWAP;
+#endif
+	    break;
 
 	 case ' ' :
 	 case '\t': break;
@@ -644,48 +686,3 @@ char *rp;
 
    return(flag);
 }
-
-
-static char *
-_normalize_path(path)/*
-=======================
-   Normalize the given path unless it contains a $ indicating a dynamic
-   prerequisite.
-   Special case: For absolute DOSish paths under cygwin a cygwin API
-   function is used to normalize the path optherwise Clean_path() is used.
-
-   Note, the returned path is built in a static buffer, if it is to be used
-   later a copy should be created. */
-
-char *path;
-{
-   static char *cpath = NIL(char);
-
-   if ( !cpath && ( (cpath = MALLOC( PATH_MAX, char)) == NIL(char) ) )
-      No_ram();
-
-   /* If there is a $ in the path this can either mean a '$' character in
-    * a target definition or a dynamic macro expression in a prerequisite
-    * list. As dynamic macro expression must not be normalized and is
-    * indistinguishable from a literal $ characters at this point we skip
-    * the normalization if a $ is found.  */
-   if( strchr(path, '$') ) {
-      return path;
-   }
-
-#if __CYGWIN__
-   /* Use cygwin function to convert a DOS path to a POSIX path. */
-   if( *path && path[1] == ':' && isalpha(*path) ) {
-      cygwin_conv_to_posix_path(path, cpath);
-   }
-   else
-#endif
-   {
-      strcpy( cpath, path );
-      Clean_path( cpath );
-   }
-
-   DB_PRINT( "path", ("normalized: %s", cpath ));
-
-   return cpath;
-}
