@@ -1,6 +1,6 @@
 /* $RCSfile: extern.h,v $
--- $Revision: 1.9.12.4 $
--- last change: $Author: vq $ $Date: 2006/11/12 05:06:38 $
+-- $Revision: 1.11.4.3 $
+-- last change: $Author: vq $ $Date: 2007/09/23 22:05:02 $
 --
 -- SYNOPSIS
 --      External declarations for dmake functions.
@@ -105,15 +105,31 @@
 char *strlwr(char *p);
 #endif
 
+/* from function.c */
+char *exec_normpath(char *args);
+
+/* from make.c */
+void Unmake(CELLPTR cp);
+
 /* from path.c */
 void Clean_path(char *path);
+char *normalize_path(char *path);
 
+/* from sysintf.c */
+/* cygdospath()/DO_WINPATH() are only needed for the .WINPATH attribute
+ * on cygwin. */
+#if __CYGWIN__
+char *cygdospath(char *src, int winpath);
+# define DO_WINPATH(p) cygdospath(p, UseWinpath)
+#else
+# define DO_WINPATH(p) p
+#endif
 
 
 /* Define some usefull macros. This is done here and not in config.h
  * to keep this changes usefull even when not using the autotools based
  * build, i.e. using config.h files that are local to the architecture. */
-#if defined(_WIN32) || defined(__CYGWIN__) || defined(MSDOS) || defined(OS2)
+#if defined(_WIN32) || defined(__CYGWIN__) || defined(MSDOS) || defined(OS2) || defined(__EMX__)
 #  define HAVE_DRIVE_LETTERS 1
 #endif
 
