@@ -1,4 +1,4 @@
-/* RCS  $Id: getinp.c,v 1.7.6.1 2006/11/12 05:06:38 vq Exp $
+/* RCS  $Id: getinp.c,v 1.9.4.2 2007/09/19 00:34:30 vq Exp $
 --
 -- SYNOPSIS
 --      Handle reading of input.
@@ -305,10 +305,16 @@ int  keep;
 	       cmnd = Expand(c+2);
 	       cmnd[strlen(cmnd)-1] = '\0';	/* strip last newline */
 	       Current_target = Root;
+#if defined(MSDOS)
 	       Swap_on_exec = TRUE;
+#endif
 	       Wait_for_completion = TRUE;
-	       Do_cmnd(cmnd, FALSE, TRUE, Current_target, A_DEFAULT, TRUE);
+	       Do_cmnd(&cmnd, FALSE, TRUE, Current_target, A_DEFAULT, TRUE);
+#if defined(MSDOS)
+	       Swap_on_exec = FALSE;
+#endif
 	       Wait_for_completion = FALSE;
+	       FREE(cmnd);
 	    }
 
 	    *c = '\0';               /* a true comment so break */
@@ -534,17 +540,9 @@ _handle_conditional( opcode, tg )
 	    else
 	       cst++;
 
-	 lhs = DmStrSpn( expr, " \t" );
-	 if( !*lhs ) lhs = NIL(char);
-
-      /* Don't support C-style define keyword */
-	 if ( (lop = DmStrStr(lhs, "defined" )) !=  NIL(char) )
-	    Fatal( ".IF do not support defined " );
-	 if ( (lop = DmStrStr(lhs, "DEFINED" )) !=  NIL(char) )
-	    Fatal( ".IF do not support defined " );
-
 	 lhs = expr;
-      SCAN_WHITE( lhs );
+	 SCAN_WHITE( lhs );
+
 	 /* Parse the expression and get its logical result */
 	 if ( ((lop = DmStrStr(lhs, "||" )) !=  NIL(char)) || ((lop = DmStrStr(lhs, "&&" )) !=  NIL(char)) )
 		result = parse_complex_expression( lhs, &expr_end, opcode );
