@@ -1,4 +1,4 @@
-/* RCS  $Id: imacs.c,v 1.4.6.4 2006/11/19 05:27:24 vq Exp $
+/* RCS  $Id: imacs.c,v 1.7.2.2 2007/09/22 23:05:37 vq Exp $
 --
 -- SYNOPSIS
 --      Define default internal macros.
@@ -68,6 +68,9 @@ Create_macro_vars()
    else
 #if (_MPW)
    	  DirSepStr = ":";
+#elif defined( __EMX__)
+   /* Use '\' for OS/2 port. */
+   	  DirSepStr = "\\";
 #else
    	  DirSepStr = "/";
 #endif
@@ -86,7 +89,9 @@ Create_macro_vars()
    _set_bit_var(".NOINFER",  "", A_NOINFER );
    _set_bit_var(".SEQUENTIAL","",A_SEQ     );
    _set_bit_var(".USESHELL", "", A_SHELL   );
+   /* .SWAP (MSDOS) and .WINPATH (cygwin) share the same bit. */
    _set_bit_var(".SWAP",     "", A_SWAP    );
+   _set_bit_var(".WINPATH",  "", A_WINPATH );
    _set_bit_var(".MKSARGS",  "", A_MKSARGS );
    _set_bit_var(".IGNOREGROUP","",A_IGNOREGROUP);
 
@@ -94,11 +99,13 @@ Create_macro_vars()
 
    _set_string_var("SHELL",        "",  M_DEFAULT, &Shell       );
    _set_string_var("SHELLFLAGS",   " ", M_DEFAULT, &Shell_flags );
+   _set_string_var("SHELLCMDQUOTE","",  M_DEFAULT, &Shell_quote );
    _set_string_var("GROUPSHELL",   "",  M_DEFAULT, &GShell      );
    _set_string_var("GROUPFLAGS",   " ", M_DEFAULT, &GShell_flags);
    _set_string_var("SHELLMETAS",   "",  M_DEFAULT, &Shell_metas );
    _set_string_var("GROUPSUFFIX",  "",  M_DEFAULT, &Grp_suff    );
    _set_string_var("AUGMAKE",NIL(char), M_DEFAULT, &Augmake     );
+   _set_string_var("OOODMAKEMODE", "",  M_DEFAULT, &OOoDmMode );
    _set_string_var(".KEEP_STATE",  "",  M_DEFAULT, &Keep_state  );
    _set_string_var(".NOTABS",      "",  M_MULTI, &Notabs );
    _set_string_var(".DIRCACHE",    "y", M_DEFAULT, &UseDirCache );
@@ -110,10 +117,14 @@ Create_macro_vars()
 #endif
    _set_string_var(".DIRCACHERESPCASE", DIRCACHERESPCASEDEFAULT, M_DEFAULT, &DcacheRespCase);
 
-   _set_string_var("MAKEDIR",Get_current_dir(),M_PRECIOUS|M_NOEXPORT,&Makedir);
-   _set_string_var("MAKEVERSION", VERSION, M_DEFAULT|M_PRECIOUS, &version);
-   _set_string_var("PWD",  Makedir,  M_DEFAULT|M_NOEXPORT, &Pwd);
-   _set_string_var("TMD",  ".",      M_DEFAULT|M_NOEXPORT, &Tmd);
+   _set_string_var("MAKEDIR",Get_current_dir(),M_PRECIOUS|M_NOEXPORT,
+		   &Makedir_macval);
+   Makedir = DmStrDup(Makedir_macval); /* Later done by Def_macro(). */
+   _set_string_var("MAKEVERSION", VERSION, M_PRECIOUS, &version);
+   _set_string_var("PWD",  Makedir,  M_PRECIOUS|M_NOEXPORT, &Pwd_macval);
+   Pwd = DmStrDup(Pwd_macval); /* Later done by Def_macro(). */
+   _set_string_var("TMD",  ".",      M_PRECIOUS|M_NOEXPORT, &Tmd_macval);
+   Tmd = DmStrDup(Tmd_macval); /* Later done by _set_tmd(). */
 
    Def_macro("NULL", "", M_PRECIOUS|M_NOEXPORT|M_FLAG);
 
