@@ -1,4 +1,4 @@
-/* RCS  $Id: sysintf.h,v 1.1.1.1.156.1 2006/09/30 15:01:16 vq Exp $
+/* RCS  $Id: sysintf.h,v 1.3.2.1 2007/09/22 23:05:37 vq Exp $
 --
 -- SYNOPSIS
 --      Interfaces for sysintf.c
@@ -27,8 +27,9 @@
 #define VOID_LCACHE(l,m)
 #define Hook_std_writes(A)
 #define GETPID getpid()
-#define S_IFMT (S_IFDIR|S_IFCHR|S_IFREG)
-
+/* Disabled for CWS os2port01
+ *#define S_IFMT (S_IFDIR|S_IFCHR|S_IFREG)
+ */
 extern char * tempnam();
 extern char * getcwd();
 
@@ -42,7 +43,9 @@ extern char * getcwd();
 /*
 ** DOS interface standard items
 */
-#define	chdir(p) _dchdir(p)
+/* Disabled for CWS os2port01
+ *#define	chdir(p) _dchdir(p)
+ */
 #define CacheStat(A,B) really_dostat(A,&buf)
 
 /*
