@@ -1,6 +1,6 @@
 /* $RCSfile: parse.c,v $
--- $Revision: 1.6.6.1 $
--- last change: $Author: vq $ $Date: 2006/10/07 20:23:15 $
+-- $Revision: 1.7.6.1 $
+-- last change: $Author: vq $ $Date: 2007/08/09 01:43:17 $
 --
 -- SYNOPSIS
 --      Parse the input, and perform semantic analysis
@@ -84,10 +84,12 @@ FILE *fil;
 
 	       p = DmStrSpn( Buffer, " \t\r\n" );
                if( Set_group_attributes( p ) ) {
-                  if( rule && Group )
-                     Fatal( "Cannot mix single and group recipe lines" );
-                  else
-                     Group = TRUE;
+		  if( Group )
+		     Fatal( "New group recipe begin found within group recipe." );
+		  else if( rule )
+		     Fatal( "Cannot mix single and group recipe lines." );
+		  else
+		     Group = TRUE;
 
                   rule = TRUE;
 
