@@ -1,4 +1,4 @@
-/* RCS  $Id: runargv.c,v 1.1.1.1.156.1 2006/11/12 05:06:43 vq Exp $
+/* RCS  $Id: runargv.c,v 1.2.6.2 2007/09/19 00:34:34 vq Exp $
 --
 -- SYNOPSIS
 --      Invoke a sub process.
@@ -60,6 +60,8 @@ static  int     _running ANSI((CELLPTR));
 
 /* iz71422 changed the parameters for runargv but it (and the rest of
  * qssl) got *NOT* fixed. */
+iz81252 changed the parameters for Pack_argv() and runargv()  but this file
+did not get fixed!
 PUBLIC int
 runargv(target, ignore, group, last, shell, cmd)
 CELLPTR target;
@@ -214,7 +216,6 @@ int	status;
       Current_target = NIL(CELL);
 
       if ( _procs[i].pr_target->ce_attr & A_ERROR ) {
-	 Unlink_temp_files( _procs[i].pr_target );
 	 _procs[i].pr_last = TRUE;
 	 goto ABORT_REMAINDER_OF_RECIPE;
       }
@@ -232,7 +233,6 @@ int	status;
       if( _proc_cnt == Max_proc ) Wait_for_child( FALSE, -1 );
    }
    else {
-      Unlink_temp_files( _procs[i].pr_target );
       Handle_result(status,_procs[i].pr_ignore,_abort_flg,_procs[i].pr_target);
 
  ABORT_REMAINDER_OF_RECIPE:
