@@ -1,4 +1,4 @@
-/* RCS  $Id: dirbrk.c,v 1.1.1.1.156.1 2006/10/01 16:23:32 vq Exp $
+/* RCS  $Id: dirbrk.c,v 1.3.2.1 2007/09/22 23:05:38 vq Exp $
 --
 -- SYNOPSIS
 --      Define the directory separator string.
@@ -27,8 +27,15 @@
 
 #include "extern.h"
 
+#ifdef __EMX__
+/* os2 uses /, \, and : */
+/* FIXME: The OS/2 port most probably wants to use the HAVE_DRIVE_LETTERS
+ * macro, see extern.h. */
+char*	DirBrkStr = "/\\:";
+#else
 /* Unix only uses / */
 char*	DirBrkStr = "/";
+#endif
 
 /*
 ** Return TRUE if the name is the full specification of a path name to a file
